from cerberus import Validator
from abejacli.exceptions import InvalidConfigException, ConfigFileNotFoundError
from ruamel.yaml import YAML

yaml = YAML()
HANDLER_REGEX = '\A(([a-zA-Z_]\w+\.)+)?[a-zA-Z_]\w+:[a-zA-Z_]\w+\Z'
CONFIGFILE_NAME = "training.yaml"

training_default_configuration = """\
name: training-1
# handler: train:handler
# image: abeja-inc/minimal:0.1.0
# params:
#   param1: value1
#   parma2: value2
# datasets:
#   dataset_name1: value1
#   dataset_name2: value2
"""

default_schema = {
    'name': {'type': 'string', 'required': True},
    'handler': {'type': 'string', 'required': False, 'regex': HANDLER_REGEX},
    'image': {'type': 'string', 'required': False},
    'datasets': {
        'type': 'dict',
        'keyschema': {'type': 'string'},
        'valueschema': {'type': 'string', 'coerce': str},
        'required': False
    },
    'params': {
        'type': 'dict',
        'keyschema': {'type': 'string'},
        'valueschema': {'type': 'string'},
        'required': False
    }
}
create_version_schema = {
    'name': {'type': 'string', 'required': True},
    'handler': {'type': 'string', 'required': True, 'regex': HANDLER_REGEX},
    'image': {'type': 'string', 'required': True},
    'datasets': {
        'type': 'dict',
        'keyschema': {'type': 'string'},
        'valueschema': {'type': 'string', 'coerce': str},
        'required': True
    },
    'params': {
        'type': 'dict',
        'keyschema': {'type': 'string'},
        'valueschema': {'type': 'string'},
        'required': False
    }
}


class TrainingConfig:
    def __init__(self):
        self.config_file = training_default_configuration
        self.default_schema = default_schema
        self.create_version_schema = create_version_schema

    def write(self, name):
        data = yaml.load(self.config_file)
        data['name'] = name
        with open(CONFIGFILE_NAME, 'w') as configfile:
            yaml.dump(data, configfile)

    def read(self, schema):
        try:
            with open(CONFIGFILE_NAME, 'r') as configfile:
                config_data = yaml.load(configfile.read())
            self._validate(config_data, schema)
        except FileNotFoundError:
            raise ConfigFileNotFoundError('configuration file not found')
        return config_data

    def _validate(self, config_data, schema):
        v = Validator(schema)
        if v.validate(config_data):
            return True
        else:
            raise InvalidConfigException('configuration file validation failed: {}.'.format(v.errors))
